@@ -23,27 +23,12 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <ctype.h>
+#include <unistd.h>
 
 #include <gphoto2/gphoto2.h>
 
-#ifdef ENABLE_NLS
-#  include <libintl.h>
-#  undef _
-#  define _(String) dgettext (GETTEXT_PACKAGE, String)
-#  ifdef gettext_noop
-#    define N_(String) gettext_noop (String)
-#  else
-#    define N_(String) (String)
-#  endif
-#else
-#  define textdomain(String) (String)
-#  define gettext(String) (String)
-#  define dgettext(Domain,Message) (Message)
-#  define dcgettext(Domain,Message,Type) (Message)
-#  define bindtextdomain(Domain,Directory) (Domain)
-#  define _(String) (String)
-#  define N_(String) (String)
-#endif
+#include "libgphoto2/i18n.h"
+
 
 #include "library.h"
 #include "canon.h"
@@ -211,7 +196,7 @@ canon_serial_send (Camera *camera, const unsigned char *buf, int len, int sleep)
  * @gdev: serial port to use
  * @to:   timeout in milliseconds
  *
- * Sets the timeout, in miliseconds.
+ * Sets the timeout, in milliseconds.
  *
  */
 static void
@@ -349,8 +334,8 @@ canon_serial_recv_frame (Camera *camera, int *len)
 /**
  * canon_serial_send_packet
  * @camera: Camera object to work with
- * @type: 
- * @seq: 
+ * @type:
+ * @seq:
  * @pkt: data to send to camera
  * @len: length of data
  *
@@ -691,7 +676,7 @@ canon_serial_recv_msg (Camera *camera, unsigned char mtype, unsigned char dir, u
 		if (type == PKT_ACK && mtype == 0x3 && dir == 0x21) {
 			GP_DEBUG("ignoring ACK received while waiting for MSG");
 			return frag;
-		} 
+		}
 */
 		if (len < MSG_HDR_LEN || frag[MSG_02] != 2) {
 			GP_DEBUG ("ERROR: message format error");
@@ -1368,11 +1353,11 @@ canon_serial_ready (Camera *camera, GPContext *context)
 	 * don't actually use id_str, but serial_id_string.
 	 */
 	for (i = 0; models[i].id_str != NULL; i++) {
-		if ((models[i].serial_id_string != NULL) && 
+		if ((models[i].serial_id_string != NULL) &&
 		    !strcmp (models[i].serial_id_string, cam_id_str)) {
 			GP_DEBUG ("canon_serial_ready: Serial ID string matches '%s'",
 				  models[i].serial_id_string);
-			gp_context_status (context, _("Detected a \"%s\" aka \"%s\""), 
+			gp_context_status (context, _("Detected a \"%s\" aka \"%s\""),
 					   models[i].id_str, models[i].serial_id_string);
 			camera->pl->md = (struct canonCamModelData *) &models[i];
 			break;
@@ -1478,7 +1463,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
  * @context: context for error reporting
  *
  * This is just the serial specific part extracted from the older
- * canon_get_thumbnail() routine. 
+ * canon_get_thumbnail() routine.
  *
  * Returns: gphoto2 error code
  *
