@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301  USA
  */
-
+#define _DARWIN_C_SOURCE
 #include "config.h"
 #include <gphoto2/gphoto2-port-library.h>
 #include <gphoto2/gphoto2-port-portability.h>
@@ -44,24 +44,8 @@
 #include <gphoto2/gphoto2-port-result.h>
 #include <gphoto2/gphoto2-port-log.h>
 
-#ifdef ENABLE_NLS
-#  include <libintl.h>
-#  undef _
-#  define _(String) dgettext (GETTEXT_PACKAGE, String)
-#  ifdef gettext_noop
-#    define N_(String) gettext_noop (String)
-#  else
-#    define N_(String) (String)
-#  endif
-#else
-#  define textdomain(String) (String)
-#  define gettext(String) (String)
-#  define dgettext(Domain,Message) (Message)
-#  define dcgettext(Domain,Message,Type) (Message)
-#  define bindtextdomain(Domain,Directory) (Domain)
-#  define _(String) (String)
-#  define N_(String) (String)
-#endif
+#include "libgphoto2_port/i18n.h"
+
 
 #define CHECK(result) {int r=(result); if (r<0) return (r);}
 
@@ -118,14 +102,18 @@ static int put_8bit_le(unsigned char *data, uint8_t x) {
 static int put_string(unsigned char *data, char *str) {
 	int i;
 
-	if (strlen(str)>255)
+	if (!str) {	/* empty string, just has length 0 */
+		data[0] = 0;
+		return 1;
+	}
+	if (strlen(str)+1>255)
 		gp_log (GP_LOG_ERROR, "put_string", "string length is longer than 255 characters");
 
-	data[0] = strlen (str);
+	data[0] = strlen (str)+1;
 	for (i=0;i<data[0];i++)
 		put_16bit_le(data+1+2*i,str[i]);
 
-	return 1+strlen(str)*2;
+	return 1+(strlen(str)+1)*2;
 }
 
 static char * get_string(unsigned char *data) {
@@ -217,6 +205,7 @@ ptp_response(vcamera *cam, uint16_t code, int nparams, ...) {
 #define PTP_RC_InvalidObjectFormatCode			0x200B
 #define PTP_RC_StoreFull				0x200C
 #define PTP_RC_ObjectWriteProtected			0x200D
+#define PTP_RC_StoreReadOnly				0x200E
 #define PTP_RC_AccessDenied				0x200F
 #define PTP_RC_NoThumbnailPresent			0x2010
 #define PTP_RC_StoreNotAvailable			0x2013
@@ -351,6 +340,41 @@ struct _PTPDevicePropDesc {
 };
 typedef struct _PTPDevicePropDesc PTPDevicePropDesc;
 
+// We need these (modified) helpers from ptp.c.
+// Perhaps vcamera.c should be moved to camlibs/ptp2 for easier sharing
+// in the future.
+static void
+ptp_free_devicepropvalue(uint16_t dt, PTPPropertyValue* dpd)
+{
+	if (dt == /* PTP_DTC_STR */ 0xFFFF) {
+		free(dpd->str);
+	} else if (dt & /* PTP_DTC_ARRAY_MASK */ 0x4000) {
+		free(dpd->a.v);
+	}
+}
+
+static void
+ptp_free_devicepropdesc(PTPDevicePropDesc* dpd)
+{
+	uint16_t i;
+
+	ptp_free_devicepropvalue (dpd->DataType, &dpd->FactoryDefaultValue);
+	ptp_free_devicepropvalue (dpd->DataType, &dpd->CurrentValue);
+	switch (dpd->FormFlag) {
+	case /* PTP_DPFF_Range */ 0x01:
+		ptp_free_devicepropvalue (dpd->DataType, &dpd->FORM.Range.MinimumValue);
+		ptp_free_devicepropvalue (dpd->DataType, &dpd->FORM.Range.MaximumValue);
+		ptp_free_devicepropvalue (dpd->DataType, &dpd->FORM.Range.StepSize);
+		break;
+	case /* PTP_DPFF_Enumeration */ 0x02:
+		if (dpd->FORM.Enum.SupportedValue) {
+			for (i=0;i<dpd->FORM.Enum.NumberOfValues;i++)
+				ptp_free_devicepropvalue (dpd->DataType, dpd->FORM.Enum.SupportedValue+i);
+			free (dpd->FORM.Enum.SupportedValue);
+		}
+	}
+}
+
 static int ptp_battery_getdesc(vcamera*,PTPDevicePropDesc*);
 static int ptp_battery_getvalue(vcamera*,PTPPropertyValue*);
 static int ptp_imagesize_getdesc(vcamera*,PTPDevicePropDesc*);
@@ -394,8 +418,28 @@ struct ptp_dirent {
 static struct ptp_dirent *first_dirent = NULL;
 static uint32_t	ptp_objectid = 0;
 
+static void *read_file(struct ptp_dirent *cur) {
+	FILE *file = fopen(cur->fsname, "rb");
+	if (!file) {
+		gp_log (GP_LOG_ERROR,__FUNCTION__, "could not open %s", cur->fsname);
+		return NULL;
+	}
+	void *data = malloc(cur->stbuf.st_size);
+	if (!data) {
+		gp_log (GP_LOG_ERROR,__FUNCTION__, "could not allocate data for %s", cur->fsname);
+		return NULL;
+	}
+	if (!fread(data, cur->stbuf.st_size, 1, file)) {
+		gp_log (GP_LOG_ERROR,__FUNCTION__, "could not read data of %s", cur->fsname);
+		free(data);
+		data = NULL;
+	}
+	fclose(file);
+	return data;
+}
+
 static void
-read_directories(char *path, struct ptp_dirent *parent) {
+read_directories(const char *path, struct ptp_dirent *parent) {
 	struct ptp_dirent	*cur;
 	gp_system_dir		dir;
 	gp_system_dirent	de;
@@ -433,7 +477,7 @@ free_dirent(struct ptp_dirent *ent) {
 }
 
 static void
-read_tree(char *path) {
+read_tree(const char *path) {
 	struct	ptp_dirent *root = NULL, *dir, *dcim = NULL;
 
 	if (first_dirent)
@@ -457,7 +501,7 @@ read_tree(char *path) {
 	}
 	if (!dcim) {
 		dcim = malloc(sizeof(struct ptp_dirent));
-		dcim->name = strdup("");
+		dcim->name = strdup("DCIM");
 		dcim->fsname = strdup(path);
 		dcim->id = ptp_objectid++;
 		dcim->next = first_dirent;
@@ -476,12 +520,6 @@ ptp_nikon_setcontrolmode_write(vcamera *cam, ptpcontainer *ptp) {
 		ptp_response (cam, PTP_RC_InvalidParameter, 0);
 		return 1;
 	}
-	if (cam->session) {
-		gp_log (GP_LOG_ERROR,__FUNCTION__,"session is already open");
-		ptp_response (cam, PTP_RC_SessionAlreadyOpened, 0);
-		return 1;
-	}
-	cam->session = ptp->params[0];
 	ptp_response (cam,PTP_RC_OK,0);
 	return 1;
 }
@@ -888,27 +926,14 @@ ptp_getobjectinfo_write(vcamera *cam, ptpcontainer *ptp) {
 	if (ofc == 0x3801) {			/* We are jpeg ... look into the exif data */
 		ExifData	*ed;
 		ExifEntry	*e;
-		int		fd;
 		unsigned char	*filedata;
 
-		filedata = malloc(cur->stbuf.st_size);
-		fd =  open(cur->fsname,O_RDONLY);
-		if (fd == -1) {
-			free (filedata);
+		filedata = read_file(cur);
+		if (!filedata) {
 			free (data);
-			gp_log (GP_LOG_ERROR,__FUNCTION__, "could not open %s", cur->fsname);
 			ptp_response(cam,PTP_RC_GeneralError,0);
 			return 1;
 		}
-		if (cur->stbuf.st_size != read(fd, filedata, cur->stbuf.st_size)) {
-			free (filedata);
-			free (data);
-			close (fd);
-			gp_log (GP_LOG_ERROR,__FUNCTION__, "could not read data of %s", cur->fsname);
-			ptp_response(cam,PTP_RC_GeneralError,0);
-			return 1;
-		}
-		close (fd);
 
 		ed = exif_data_new_from_data ((unsigned char*)filedata, cur->stbuf.st_size);
 		if (ed) {
@@ -979,7 +1004,6 @@ static int
 ptp_getobject_write(vcamera *cam, ptpcontainer *ptp) {
 	unsigned char 		*data;
 	struct ptp_dirent	*cur;
-	int fd;
 
 	CHECK_SEQUENCE_NUMBER();
 	CHECK_SESSION();
@@ -995,22 +1019,11 @@ ptp_getobject_write(vcamera *cam, ptpcontainer *ptp) {
 		ptp_response(cam,PTP_RC_InvalidObjectHandle,0);
 		return 1;
 	}
-	data = malloc(cur->stbuf.st_size);
-	fd =  open(cur->fsname,O_RDONLY);
-	if (fd == -1) {
-		free (data);
-		gp_log (GP_LOG_ERROR,__FUNCTION__, "could not open %s", cur->fsname);
+	data = read_file(cur);
+	if (!data) {
 		ptp_response(cam,PTP_RC_GeneralError,0);
 		return 1;
 	}
-	if (cur->stbuf.st_size != read(fd, data, cur->stbuf.st_size)) {
-		free (data);
-		close (fd);
-		gp_log (GP_LOG_ERROR,__FUNCTION__, "could not read data of %s", cur->fsname);
-		ptp_response(cam,PTP_RC_GeneralError,0);
-		return 1;
-	}
-	close (fd);
 
 	ptp_senddata (cam, 0x1009, data, cur->stbuf.st_size);
 	free (data);
@@ -1022,7 +1035,6 @@ static int
 ptp_getthumb_write(vcamera *cam, ptpcontainer *ptp) {
 	unsigned char 		*data;
 	struct ptp_dirent	*cur;
-	int			fd;
 #ifdef HAVE_LIBEXIF
         ExifData		*ed;
 #endif
@@ -1041,22 +1053,11 @@ ptp_getthumb_write(vcamera *cam, ptpcontainer *ptp) {
 		ptp_response(cam,PTP_RC_InvalidObjectHandle,0);
 		return 1;
 	}
-	data = malloc(cur->stbuf.st_size);
-	fd =  open(cur->fsname,O_RDONLY);
-	if (fd == -1) {
-		free (data);
-		gp_log (GP_LOG_ERROR,__FUNCTION__, "could not open %s", cur->fsname);
+	data = read_file(cur);
+	if (!data) {
 		ptp_response(cam,PTP_RC_GeneralError,0);
 		return 1;
 	}
-	if (cur->stbuf.st_size != read(fd, data, cur->stbuf.st_size)) {
-		free (data);
-		close (fd);
-		gp_log (GP_LOG_ERROR,__FUNCTION__, "could not read data of %s", cur->fsname);
-		ptp_response(cam,PTP_RC_GeneralError,0);
-		return 1;
-	}
-	close (fd);
 
 #ifdef HAVE_LIBEXIF
 	ed = exif_data_new_from_data ((unsigned char*)data, cur->stbuf.st_size);
@@ -1115,6 +1116,7 @@ ptp_initiatecapture_write(vcamera *cam, ptpcontainer *ptp) {
 		return 1;
 	}
 
+	/* just search for first jpeg we find in the tree, we will use this to send back as the captured image */
 	cur = first_dirent;
 	while (cur) {
 		if (strstr (cur->name, ".jpg") || strstr (cur->name, ".JPG"))
@@ -1123,34 +1125,17 @@ ptp_initiatecapture_write(vcamera *cam, ptpcontainer *ptp) {
 	}
 	if (!cur) {
 		gp_log (GP_LOG_ERROR,__FUNCTION__, "I do not have a JPG file in the store, can not proceed");
-		ptp_response (cam, PTP_RC_GeneralError, 0);
+		ptp_response (cam, PTP_RC_StoreNotAvailable, 0);
 		return 1;
 	}
+	/* identify the DCIM dir, so we can attach a virtual xxxGPHOT directory to virtually store the new picture in */
 	dir = first_dirent;
 	while (dir) {
 		if (!strcmp(dir->name,"DCIM") && dir->parent && !dir->parent->id)
 			dcim = dir;
 		dir = dir->next;
 	}
-
-	cur = first_dirent;
-	while (cur) {
-		if (strstr (cur->name, ".jpg") || strstr (cur->name, ".JPG"))
-			break;
-		cur = cur->next;
-	}
-	if (!cur) {
-		gp_log (GP_LOG_ERROR,__FUNCTION__, "I do not have a JPG file in the store, can not proceed");
-		ptp_response (cam, PTP_RC_GeneralError, 0);
-		return 1;
-	}
-	dir = first_dirent;
-	while (dir) {
-		if (!strcmp(dir->name,"DCIM") && dir->parent && !dir->parent->id)
-			dcim = dir;
-		dir = dir->next;
-	}
-	/* nnnGPHOT directories, where nnn is 100-999. (See DCIM standard.) */
+	/* find the nnnGPHOT directories, where nnn is 100-999. (See DCIM standard.) */
 	sprintf(buf, "%03dGPHOT", 100 + ((capcnt / 100) % 900));
 	dir = first_dirent;
 	while (dir) {
@@ -1158,6 +1143,7 @@ ptp_initiatecapture_write(vcamera *cam, ptpcontainer *ptp) {
 			break;
 		dir = dir->next;
 	}
+	/* if not yet found, create the virtual /DCIM/xxxGPHOT/ directory. */
 	if (!dir) {
 		dir 		= malloc (sizeof(struct ptp_dirent));
 		dir->id		= ++ptp_objectid;
@@ -1351,6 +1337,8 @@ ptp_getdevicepropdesc_write(vcamera *cam, ptpcontainer *ptp) {
 		break;
 	}
 
+	ptp_free_devicepropdesc(&desc);
+
 	ptp_senddata (cam, 0x1014, data, x);
 	free (data);
 	ptp_response (cam, PTP_RC_OK, 0);
@@ -1809,6 +1797,7 @@ static int vcam_exit(vcamera* cam) {
 }
 
 static int vcam_open(vcamera* cam, const char *port) {
+#ifdef FUZZING
 	char *s = strchr(port,':');
 
 	if (s) {
@@ -1817,20 +1806,34 @@ static int vcam_open(vcamera* cam, const char *port) {
 			cam->fuzzmode = FUZZMODE_PROTOCOL;
 		} else {
 			cam->fuzzf = fopen(s+1,"rb");
+#ifndef FUZZ_PTP
+			/* first 4 byte are vendor and product USB id */
+			if (cam->fuzzf)
+				fseek (cam->fuzzf, 4, SEEK_SET);
+#endif
 			cam->fuzzpending = 0;
 			cam->fuzzmode = FUZZMODE_NORMAL;
 		}
-		if (cam->fuzzf == NULL) perror(s+1);
+		/* in virtual camera mode we have no fuzzing
+		if (cam->fuzzf == NULL) {
+			perror(s+1);
+		}
+		*/
 	}
+#endif
 	return GP_OK;
 }
 
 static int vcam_close(vcamera* cam) {
+#ifdef FUZZING
 	if (cam->fuzzf) {
 		fclose (cam->fuzzf);
 		cam->fuzzf = NULL;
 		cam->fuzzpending = 0;
 	}
+#endif
+	free (cam->inbulk);
+	free (cam->outbulk);
 	return GP_OK;
 }
 
@@ -1936,6 +1939,8 @@ static int
 vcam_read(vcamera*cam, int ep, unsigned char *data, int bytes) {
 	unsigned int	toread = bytes;
 
+#ifdef FUZZING
+	/* here we are not using the virtual ptp camera, but just read from a file, or write to it. */
 	if (cam->fuzzf) {
 		unsigned int hasread;
 
@@ -1944,6 +1949,7 @@ vcam_read(vcamera*cam, int ep, unsigned char *data, int bytes) {
 			fwrite(cam->inbulk, 1, toread, cam->fuzzf);
 			/* fallthrough */
 		} else {
+#ifdef FUZZ_PTP
 			/* for reading fuzzer data */
 			if (cam->fuzzpending) {
 				toread = cam->fuzzpending;
@@ -1953,7 +1959,7 @@ vcam_read(vcamera*cam, int ep, unsigned char *data, int bytes) {
 			} else {
 				hasread = fread (data, 1, 4, cam->fuzzf);
 				if (hasread != 4)
-					return 0;
+					return GP_ERROR_IO_READ;
 
 				toread = data[0] | (data[1]<<8) | (data[2]<<16) | (data[3]<<24);
 
@@ -1970,6 +1976,12 @@ vcam_read(vcamera*cam, int ep, unsigned char *data, int bytes) {
 
 				hasread += 4; /* readd size */
 			}
+#else
+			/* just return a blob of data in generic fuzzing */
+			hasread = fread (data, 1, bytes, cam->fuzzf);
+			if (!hasread && feof(cam->fuzzf))
+				return GP_ERROR_IO_READ;
+#endif
 #if 0
 			for (i=0;i<toread;i++)
 				data[i] ^= cam->inbulk[i];
@@ -1979,6 +1991,7 @@ vcam_read(vcamera*cam, int ep, unsigned char *data, int bytes) {
 			return toread;
 		}
 	}
+#endif /* FUZZING */
 
 	/* Emulated PTP camera stuff */
 
@@ -2073,9 +2086,14 @@ vcam_readint(vcamera*cam, unsigned char *data, int bytes, int timeout) {
 	struct ptp_interrupt	*pint;
 
 	if (!first_interrupt) {
+#ifdef FUZZING
 #ifndef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
 		usleep (timeout*1000);
 #endif
+		/* this emulates plugged out devices during fuzzing */
+		if (cam->fuzzf && feof(cam->fuzzf))
+			return GP_ERROR_IO;
+#endif /* FUZZING */
 		return GP_ERROR_TIMEOUT;
 	}
 	gettimeofday (&now, NULL);
@@ -2121,7 +2139,8 @@ vcamera_new(vcameratype type) {
 	cam = calloc(1,sizeof(vcamera));
 	if (!cam) return NULL;
 
-	read_tree(VCAMERADIR);
+	const char *vcameradir_env = getenv("VCAMERADIR");
+	read_tree(vcameradir_env != NULL ? vcameradir_env : VCAMERADIR);
 
 	cam->init = vcam_init;
 	cam->exit = vcam_exit;
